import sys
from typing import Dict, Final, Pattern, Tuple

if sys.version_info < (3, 8):
    from typing_extensions import Literal
else:
    from typing import Literal

GET_ITERATOR_CHUNK_SIZE: Final[int] = ...

MULTI: Literal["multi"] = ...
SINGLE: Literal["single"] = ...
CURSOR: Literal["cursor"] = ...
NO_RESULTS: Literal["no results"] = ...

ORDER_PATTERN: Pattern = ...
ORDER_DIR: Dict[str, Tuple[str, str]] = ...

INNER: Literal["INNER JOIN"] = ...
LOUTER: Literal["LEFT OUTER JOIN"] = ...
